@@ -1,15 +1,17 @@
+import logging
 import random
 import re
-import logging
 from io import BytesIO
+from urllib.parse import parse_qsl, urlparse
 
 from streamlink import PluginError
 from streamlink.packages.flashmedia import AMFMessage, AMFPacket
 from streamlink.packages.flashmedia.types import AMF3ObjectBase
 from streamlink.plugin import Plugin
-from streamlink.plugin.api import validate, useragents
+from streamlink.plugin.api import useragents, validate
 from streamlink.stream import HLSStream, HTTPStream, RTMPStream
-from streamlink.compat import urlparse, parse_qsl
+
+log = logging.getLogger(__name__)
 
 
 @AMF3ObjectBase.register("com.brightcove.experience.ViewerExperienceRequest")
@@ -37,7 +39,7 @@ class ContentOverride(AMF3ObjectBase):
         self.contentRefId = contentRefId
 
 
-class BrightcovePlayer(object):
+class BrightcovePlayer:
     player_page = "http://players.brightcove.net/{account_id}/{player_id}/index.html"
     api_url = "https://edge.api.brightcove.com/playback/v1/"
     amf_broker = "http://c.brightcove.com/services/messagebroker/amf"
@@ -60,8 +62,7 @@ class BrightcovePlayer(object):
 
     def __init__(self, session, account_id, player_id="default_default"):
         self.session = session
-        self.logger = logging.getLogger("streamlink.plugins.brightcove")
-        self.logger.debug("Creating player for account {0} (player_id={1})", account_id, player_id)
+        log.debug("Creating player for account {0} (player_id={1})".format(account_id, player_id))
         self.account_id = account_id
         self.player_id = player_id
 
@@ -96,9 +97,9 @@ class BrightcovePlayer(object):
         return policy_key
 
     def get_streams(self, video_id):
-        self.logger.debug("Finding streams for video: {0}", video_id)
+        log.debug("Finding streams for video: {0}".format(video_id))
         policy_key = self.policy_key(video_id)
-        self.logger.debug("Found policy key: {0}", policy_key)
+        log.debug("Found policy key: {0}".format(policy_key))
         data = self.video_info(video_id, policy_key)
         headers = {"Referer": self.player_url(video_id)}
 
@@ -112,8 +113,7 @@ class BrightcovePlayer(object):
                 q = "live"
             if ((source.get("type") == "application/x-mpegURL" and source.get("src"))
                     or (source.get("src") and ".m3u8" in source.get("src"))):
-                for s in HLSStream.parse_variant_playlist(self.session, source.get("src"), headers=headers).items():
-                    yield s
+                yield from HLSStream.parse_variant_playlist(self.session, source.get("src"), headers=headers).items()
             elif source.get("app_name"):
                 s = RTMPStream(self.session,
                                {"rtmp": source.get("app_name"),
