@@ -3,24 +3,22 @@ import errno
 import json
 import logging
 import re
-import websocket
-
 from collections import deque, namedtuple
 from random import randint
 from socket import error as SocketError
-from threading import Thread, Event
+from threading import Event, Thread
 from time import sleep
+from urllib.parse import unquote_plus, urljoin, urlparse, urlunparse
+
+import websocket
 
-from streamlink.compat import range, urljoin, urlunparse
 from streamlink.exceptions import PluginError, StreamError
-from streamlink.plugin import Plugin, PluginArguments, PluginArgument
+from streamlink.plugin import Plugin, PluginArgument, PluginArguments
 from streamlink.plugin.api import useragents, validate
-from streamlink.stream import Stream
 from streamlink.stream.dash_manifest import sleep_until, utc
 from streamlink.stream.flvconcat import FLVTagConcat
-from streamlink.stream.segmented import (
-    SegmentedStreamReader, SegmentedStreamWriter, SegmentedStreamWorker
-)
+from streamlink.stream.segmented import (SegmentedStreamReader, SegmentedStreamWorker, SegmentedStreamWriter)
+from streamlink.stream.stream import Stream
 from streamlink.utils import parse_json
 
 log = logging.getLogger(__name__)
@@ -33,7 +31,7 @@ class ModuleInfoNoStreams(Exception):
     pass
 
 
-class UHSClient(object):
+class UHSClient:
     """
     API Client, reverse engineered by observing the interactions
     between the web browser and the ustream servers.
@@ -57,6 +55,7 @@ class UHSClient(object):
         self._app_version = options.pop("app_version", self.APP_VERSION)
         self._cluster = options.pop("cluster", "live")
         self._password = options.pop("password")
+        self._proxy_url = options.pop("proxy")
         self._ws = None
 
     @property
@@ -79,11 +78,32 @@ class UHSClient(object):
         self._cluster = cluster
         self.reconnect()
 
+    @staticmethod
+    def parse_proxy_url(purl):
+        proxy_options = {}
+        if purl:
+            p = urlparse(purl)
+            proxy_options['proxy_type'] = p.scheme
+            proxy_options['http_proxy_host'] = p.hostname
+            if p.port:
+                proxy_options['http_proxy_port'] = p.port
+            if p.username:
+                proxy_options['http_proxy_auth'] = (unquote_plus(p.username), unquote_plus(p.password or ""))
+        return proxy_options
+
     def connect(self):
-        log.debug("Connecting to {0}".format(self.host))
+        proxy_options = self.parse_proxy_url(self._proxy_url)
+        if proxy_options.get('http_proxy_host'):
+            log.debug("Connecting to {0} via proxy ({1}://{2}:{3})".format(self.host,
+                                                                           proxy_options.get('proxy_type') or "http",
+                                                                           proxy_options.get('http_proxy_host'),
+                                                                           proxy_options.get('http_proxy_port') or 80))
+        else:
+            log.debug("Connecting to {0}".format(self.host))
         self._ws = websocket.create_connection(self.host,
                                                header=["User-Agent: {0}".format(useragents.CHROME)],
-                                               origin="https://www.ustream.tv")
+                                               origin="https://www.ustream.tv",
+                                               **proxy_options)
 
         args = dict(type="viewer",
                     appId=self._app_id,
@@ -158,7 +178,7 @@ class UHSStreamWriter(SegmentedStreamWriter):
                                          timeout=self.timeout,
                                          exception=StreamError)
         except StreamError as err:
-            log.error("Failed to open chunk {0}: {1}", chunk.num, err)
+            log.error(f"Failed to open chunk {chunk.num}: {err}")
             return self.fetch(chunk, retries - 1)
 
     def write(self, chunk, res, chunk_size=8192):
@@ -169,9 +189,9 @@ class UHSStreamWriter(SegmentedStreamWriter):
                 if self.closed:
                     return
             else:
-                log.debug("Download of chunk {0} complete", chunk.num)
-        except IOError as err:
-            log.error("Failed to read chunk {0}: {1}", chunk.num, err)
+                log.debug(f"Download of chunk {chunk.num} complete")
+        except OSError as err:
+            log.error(f"Failed to read chunk {chunk.num}: {err}")
 
 
 class UHSStreamWorker(SegmentedStreamWorker):
@@ -185,8 +205,7 @@ class UHSStreamWorker(SegmentedStreamWorker):
 
         self.process_chunks()
         if self.chunks:
-            log.debug("First Chunk: {0}; Last Chunk: {1}",
-                      self.chunks[0].num, self.chunks[-1].num)
+            log.debug(f"First Chunk: {self.chunks[0].num}; Last Chunk: {self.chunks[-1].num}")
 
     def process_chunks(self):
         chunk_data = []
@@ -223,7 +242,7 @@ class UHSStreamWorker(SegmentedStreamWorker):
     def iter_segments(self):
         while not self.closed:
             for chunk in filter(self.valid_chunk, self.chunks):
-                log.debug("Adding chunk {0} to queue", chunk.num)
+                log.debug(f"Adding chunk {chunk.num} to queue")
                 yield chunk
                 # End of stream
                 if self.closed:
@@ -235,7 +254,7 @@ class UHSStreamWorker(SegmentedStreamWorker):
                 try:
                     self.process_chunks()
                 except StreamError as err:
-                    log.warning("Failed to process module info: {0}", err)
+                    log.warning(f"Failed to process module info: {err}")
 
 
 class UHSStreamReader(SegmentedStreamReader):
@@ -293,7 +312,7 @@ class UHSStream(Stream):
                 if not cmd_args:
                     continue
                 if cmd_args["cmd"] == "warning":
-                    log.warning("{code}: {message}", **cmd_args["args"])
+                    log.warning(f"{cmd_args['args']['code']}: {cmd_args['args']['message']}")
                 if cmd_args["cmd"] == "moduleInfo":
                     data = self.handle_module_info(cmd_args["args"])
                     if data:
@@ -304,6 +323,7 @@ class UHSStream(Stream):
             def _stopper(*args, **kwargs):
                 self.stop()
                 return f(*args, **kwargs)
+
             return _stopper
 
         def handle_module_info(self, args):
@@ -316,7 +336,7 @@ class UHSStream(Stream):
                                      datetime.datetime.now(tz=utc))
 
     def __init__(self, session, api, first_chunk_data, template_url):
-        super(UHSStream, self).__init__(session)
+        super().__init__(session)
         self.session = session
         self.poller = self.APIPoller(api)
         self.poller.setDaemon(True)
@@ -336,7 +356,7 @@ class UHSStream(Stream):
 
 class UStreamTV(Plugin):
     url_re = re.compile(r"""(?x)
-    https?://(www\.)?ustream\.tv
+    https?://(?:(www\.)?ustream\.tv|video\.ibm\.com)
         (?:
             (/embed/|/channel/id/)(?P<channel_id>\d+)
         )?
@@ -411,8 +431,8 @@ class UStreamTV(Plugin):
                     raise PluginError("Stream format is not supported: {0}".format(
                         ", ".join(data["streamFormats"].keys())))
             elif "stream" in arg and arg["stream"]["contentAvailable"] is False:
-                    log.error("This stream is currently offline")
-                    raise ModuleInfoNoStreams
+                log.error("This stream is currently offline")
+                raise ModuleInfoNoStreams
 
         return res
 
@@ -432,9 +452,13 @@ class UStreamTV(Plugin):
     def _get_streams(self):
         media_id, application = self._get_media_app()
         if media_id:
-            api = UHSClient(media_id, application, referrer=self.url, cluster="live", password=self.get_option("password"))
-            log.debug("Connecting to UStream API: media_id={0}, application={1}, referrer={2}, cluster={3}",
-                      media_id, application, self.url, "live")
+            api = UHSClient(media_id, application,
+                            referrer=self.url,
+                            cluster="live",
+                            password=self.get_option("password"),
+                            proxy=self.session.get_option("http-proxy"))
+            log.debug(f"Connecting to UStream API: "
+                      f"media_id={media_id}, application={application}, referrer={self.url}, cluster=live")
             api.connect()
 
             streams_data = {}
