@@ -1,17 +1,18 @@
 import argparse
+import numbers
 import re
 from string import printable
 from textwrap import dedent
 
-from streamlink import logger
+from streamlink import __version__ as streamlink_version, logger
 from streamlink.utils.args import (
     boolean, comma_list, comma_list_filter, filesize, keyvalue, num
 )
 from streamlink.utils.times import hours_minutes_seconds
-from .constants import (
-    LIVESTREAMER_VERSION, STREAM_PASSTHROUGH, DEFAULT_PLAYER_ARGUMENTS, DEFAULT_STREAM_METADATA, SUPPORTED_PLAYERS
+from streamlink_cli.constants import (
+    DEFAULT_STREAM_METADATA, PLAYER_ARGS_INPUT_DEFAULT, PLAYER_ARGS_INPUT_FALLBACK, STREAM_PASSTHROUGH, SUPPORTED_PLAYERS
 )
-from .utils import find_default_player
+from streamlink_cli.utils import find_default_player
 
 _printable_re = re.compile(r"[{0}]".format(printable))
 _option_re = re.compile(r"""
@@ -40,9 +41,68 @@ class ArgumentParser(argparse.ArgumentParser):
 
         name, value = option.group("name", "value")
         if name and value:
-            yield u"--{0}={1}".format(name, value)
+            yield f"--{name}={value}"
         elif name:
-            yield u"--{0}".format(name)
+            yield f"--{name}"
+
+    def _match_argument(self, action, arg_strings_pattern):
+        # - https://github.com/streamlink/streamlink/issues/971
+        # - https://bugs.python.org/issue9334
+
+        # match the pattern for this action to the arg strings
+        nargs_pattern = self._get_nargs_pattern(action)
+        match = argparse._re.match(nargs_pattern, arg_strings_pattern)
+
+        # if no match, see if we can emulate optparse and return the
+        # required number of arguments regardless of their values
+        if match is None:
+            nargs = action.nargs if action.nargs is not None else 1
+            if isinstance(nargs, numbers.Number) and len(arg_strings_pattern) >= nargs:
+                return nargs
+
+        # raise an exception if we weren't able to find a match
+        if match is None:
+            nargs_errors = {
+                None: argparse._('expected one argument'),
+                argparse.OPTIONAL: argparse._('expected at most one argument'),
+                argparse.ONE_OR_MORE: argparse._('expected at least one argument'),
+            }
+            default = argparse.ngettext('expected %s argument',
+                                        'expected %s arguments',
+                                        action.nargs) % action.nargs
+            msg = nargs_errors.get(action.nargs, default)
+            raise argparse.ArgumentError(action, msg)
+
+        # return the number of arguments matched
+        return len(match.group(1))
+
+    # fix `--help` not including nested argument groups
+    def format_help(self):
+        formatter = self._get_formatter()
+
+        # usage
+        formatter.add_usage(self.usage, self._actions,
+                            self._mutually_exclusive_groups)
+
+        # description
+        formatter.add_text(self.description)
+
+        def format_group(group):
+            # positionals, optionals and user-defined groups
+            for action_group in group._action_groups:
+                formatter.start_section(action_group.title)
+                formatter.add_text(action_group.description)
+                formatter.add_arguments(action_group._group_actions)
+                format_group(action_group)
+                formatter.end_section()
+
+        format_group(self)
+
+        # epilog
+        formatter.add_text(self.epilog)
+
+        # determine help from format above
+        return formatter.format_help()
 
 
 class HelpFormatter(argparse.RawDescriptionHelpFormatter):
@@ -130,7 +190,7 @@ def build_parser():
     general.add_argument(
         "-V", "--version",
         action="version",
-        version="%(prog)s {0}".format(LIVESTREAMER_VERSION),
+        version=f"%(prog)s {streamlink_version}",
         help="""
         Show version number and exit.
         """
@@ -243,14 +303,6 @@ def build_parser():
         Default is system locale.
         """
     )
-    general.add_argument(
-        "--twitch-oauth-authenticate",
-        action="store_true",
-        help="""
-        Open a web browser where you can grant Streamlink access to your Twitch
-        account which creates a token for use with --twitch-oauth-token.
-        """
-    )
 
     player = parser.add_argument_group("Player options")
     player.add_argument(
@@ -288,33 +340,36 @@ def build_parser():
     player.add_argument(
         "-a", "--player-args",
         metavar="ARGUMENTS",
-        default=DEFAULT_PLAYER_ARGUMENTS,
-        help="""
+        default="",
+        help=f"""
         This option allows you to customize the default arguments which are put
         together with the value of --player to create a command to execute.
-        Unlike the --player parameter, custom player arguments will not be logged.
 
-        This value can contain formatting variables surrounded by curly braces,
+        It's usually enough to only use --player instead of this unless you need
+        to add arguments after the player's input argument or if you don't want
+        any of the player arguments to be logged.
+
+        The value can contain formatting variables surrounded by curly braces,
         {{ and }}. If you need to include a brace character, it can be escaped
         by doubling, e.g. {{{{ and }}}}.
 
         Formatting variables available:
 
-        {{filename}}
-            This is the filename that the player will use. It's usually "-"
-            (stdin), but can also be a URL or a file depending on the options
-            used.
+        {{{PLAYER_ARGS_INPUT_DEFAULT}}}
+            This is the input that the player will use. For standard input (stdin),
+            it is ``-``, but it can also be a URL, depending on the options used.
 
-        It's usually enough to use --player instead of this unless you need to
-        add arguments after the filename.
-
-        Default is "{0}".
+        {{{PLAYER_ARGS_INPUT_FALLBACK}}}
+            The old fallback variable name with the same functionality.
 
         Example:
 
-          %(prog)s -p vlc -a "--play-and-exit {{filename}}" <url> [stream]
+          %(prog)s -p vlc -a "--play-and-exit {{{PLAYER_ARGS_INPUT_DEFAULT}}}" <url> [stream]
 
-        """.format(DEFAULT_PLAYER_ARGUMENTS)
+        Note: When neither of the variables are found, ``{{{PLAYER_ARGS_INPUT_DEFAULT}}}``
+        will be appended to the whole parameter value, to ensure that the player
+        always receives an input argument.
+        """
     )
     player.add_argument(
         "-v", "--verbose-player",
@@ -440,7 +495,7 @@ def build_parser():
             inserted inside your --title string.
 
             A full list of the format codes mpv uses is available here:
-            https://mpv.io/manual/stable/#property-expansion
+            https://mpv.io/manual/stable/#property-list
 
         Formatting variables available to use in --title:
 
@@ -496,6 +551,14 @@ def build_parser():
         When using -o or -r, always write to file even if it already exists.
         """
     )
+    output.add_argument(
+        "--force-progress",
+        action="store_true",
+        help="""
+        When using -o or -r,
+        show the download progress bar even if there is no terminal.
+        """
+    )
     output.add_argument(
         "-O", "--stdout",
         action="store_true",
@@ -555,6 +618,13 @@ def build_parser():
         and can be useful if set in a config file.
         """
     )
+    stream.add_argument(
+        "--stream-url",
+        action="store_true",
+        help="""
+        If possible, translate the resolved stream to a URL and print it.
+        """
+    )
     stream.add_argument(
         "--retry-streams",
         metavar="DELAY",
@@ -703,6 +773,13 @@ def build_parser():
         Default is 3.
         """
     )
+    transport.add_argument(
+        "--hls-segment-stream-data",
+        action="store_true",
+        help="""
+        Immediately write segment data into output buffer while downloading.
+        """
+    )
     transport.add_argument(
         "--hls-segment-attempts",
         type=num(int, min=0),
@@ -725,6 +802,20 @@ def build_parser():
         Default is 3.
         """
     )
+    transport.add_argument(
+        "--hls-playlist-reload-time",
+        metavar="TIME",
+        help="""
+        Set a custom HLS playlist reload time value, either in seconds
+        or by using one of the following keywords:
+
+            segment: The duration of the last segment in the current playlist
+            live-edge: The sum of segment durations of the live edge value minus one
+            default: The playlist's target duration metadata
+
+        Default is default.
+        """
+    )
     transport.add_argument(
         "--hls-segment-threads",
         type=num(int, max=10),
@@ -750,16 +841,13 @@ def build_parser():
         metavar="NAMES",
         type=comma_list,
         help="""
-        A comma-delimited list of segment names that will not be fetched.
+        A comma-delimited list of segment names that will get filtered out.
 
         Example: --hls-segment-ignore-names 000,001,002
 
         This will ignore every segment that ends with 000.ts, 001.ts and 002.ts
 
         Default is None.
-
-        Note: The --hls-timeout must be increased, to a time that is longer than
-        the ignored break.
         """
     )
     transport.add_argument(
@@ -770,7 +858,16 @@ def build_parser():
         URI to segment encryption key. If no URI is specified, the URI contained
         in the segments will be used.
 
-        Example: --hls-segment-key-uri "https://example.com/hls/encryption_key"
+        URI can be templated using the following variables, which will be
+        replaced with its respective part from the source segment URI:
+
+          {url} {scheme} {netloc} {path} {query}
+
+        Examples:
+
+          --hls-segment-key-uri "https://example.com/hls/encryption_key"
+          --hls-segment-key-uri "{scheme}://1.2.3.4{path}{query}"
+          --hls-segment-key-uri "{scheme}://{netloc}/custom/path/to/key"
 
         Default is None.
         """
@@ -867,7 +964,7 @@ def build_parser():
         processing (such as HDS) to avoid unnecessary background processing.
         """)
     transport.add_argument(
-        "--rtmp-proxy", "--rtmpdump-proxy",
+        "--rtmp-proxy",
         metavar="PROXY",
         help="""
         A SOCKS proxy that RTMP streams will use.
@@ -876,7 +973,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--rtmp-rtmpdump", "--rtmpdump",
+        "--rtmp-rtmpdump",
         metavar="FILENAME",
         help="""
         RTMPDump is used to access RTMP streams. You can specify the
@@ -885,6 +982,7 @@ def build_parser():
         Example: "/usr/local/bin/rtmpdump"
         """
     )
+    transport.add_argument("--rtmpdump", help=argparse.SUPPRESS)
     transport.add_argument(
         "--rtmp-timeout",
         type=num(float, min=0),
@@ -948,14 +1046,7 @@ def build_parser():
         Default is 60.0.
         """)
     transport.add_argument(
-        "--stream-url",
-        action="store_true",
-        help="""
-        If possible, translate the stream to a URL and print it.
-        """
-    )
-    transport.add_argument(
-        "--subprocess-cmdline", "--cmdline", "-c",
+        "--subprocess-cmdline",
         action="store_true",
         help="""
         Print the command-line used internally to play the stream.
@@ -964,7 +1055,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--subprocess-errorlog", "--errorlog", "-e",
+        "--subprocess-errorlog",
         action="store_true",
         help="""
         Log possible errors from internal subprocesses to a temporary file. The
@@ -974,7 +1065,7 @@ def build_parser():
         """
     )
     transport.add_argument(
-        "--subprocess-errorlog-path", "--errorlog-path",
+        "--subprocess-errorlog-path",
         type=str,
         metavar="PATH",
         help="""
@@ -1010,6 +1101,16 @@ def build_parser():
         Path to write the output from the ffmpeg console.
         """
     )
+    transport.add_argument(
+        "--ffmpeg-fout",
+        type=str,
+        metavar="OUTFORMAT",
+        help="""
+        Set output file format.
+
+        Example: "mpegts"
+        """
+    )
     transport.add_argument(
         "--ffmpeg-video-transcode",
         metavar="CODEC",
@@ -1032,13 +1133,38 @@ def build_parser():
         Example: "aac"
         """
     )
+    transport.add_argument(
+        "--ffmpeg-copyts",
+        action="store_true",
+        help="""
+        Forces the -copyts ffmpeg option and does not remove
+        the initial start time offset value.
+        """
+    )
+    transport.add_argument(
+        "--ffmpeg-start-at-zero",
+        action="store_true",
+        help="""
+        Enable the -start_at_zero ffmpeg option when using copyts.
+        """
+    )
+    transport.add_argument(
+        "--mux-subtitles",
+        action="store_true",
+        help="""
+        Automatically mux available subtitles into the output stream.
+
+        Needs to be supported by the used plugin.
+        """
+    )
 
     http = parser.add_argument_group("HTTP options")
     http.add_argument(
         "--http-proxy",
         metavar="HTTP_PROXY",
         help="""
-        A HTTP proxy to use for all HTTP requests.
+        A HTTP proxy to use for all HTTP requests, including WebSocket connections.
+        By default this proxy will be used for all HTTPS requests too.
 
         Example: "http://hostname:port/"
         """
@@ -1142,27 +1268,6 @@ def build_parser():
         """
     )
 
-    # Deprecated options
-    http.add_argument(
-        "--http-cookies",
-        metavar="COOKIES",
-        help=argparse.SUPPRESS
-    )
-    http.add_argument(
-        "--http-headers",
-        metavar="HEADERS",
-        help=argparse.SUPPRESS
-    )
-    http.add_argument(
-        "--http-query-params",
-        metavar="PARAMS",
-        help=argparse.SUPPRESS
-    )
-    general.add_argument(
-        "--no-version-check",
-        action="store_true",
-        help=argparse.SUPPRESS
-    )
     return parser
 
 
