@@ -3,38 +3,33 @@ import errno
 import logging
 import os
 import platform
-from collections import OrderedDict
-from gettext import gettext
-
-import requests
-import sys
 import signal
-
+import sys
+from collections import OrderedDict
 from contextlib import closing
 from distutils.version import StrictVersion
 from functools import partial
+from gettext import gettext
 from itertools import chain
-from socks import __version__ as socks_version
 from time import sleep
+
+import requests
+from socks import __version__ as socks_version
 from websocket import __version__ as websocket_version
 
-from streamlink import __version__ as streamlink_version
-from streamlink import (Streamlink, StreamError, PluginError,
-                        NoPluginError)
+import streamlink.logger as logger
+from streamlink import NoPluginError, PluginError, StreamError, Streamlink, __version__ as streamlink_version
 from streamlink.cache import Cache
 from streamlink.exceptions import FatalPluginError
-from streamlink.stream import StreamProcess
 from streamlink.plugin import PluginOptions
-from streamlink.utils import LazyFormatter
-
-import streamlink.logger as logger
-from .argparser import build_parser
-from .compat import stdout, is_win32
-from streamlink.utils.encoding import maybe_decode, get_filesystem_encoding
-from .console import ConsoleOutput, ConsoleUserInputRequester
-from .constants import CONFIG_FILES, PLUGINS_DIR, STREAM_SYNONYMS, DEFAULT_STREAM_METADATA
-from .output import FileOutput, PlayerOutput
-from .utils import NamedPipe, HTTPServer, ignored, progress, stream_to_url
+from streamlink.stream import StreamProcess
+from streamlink.utils import LazyFormatter, NamedPipe
+from streamlink_cli.argparser import build_parser
+from streamlink_cli.compat import is_win32, stdout
+from streamlink_cli.console import ConsoleOutput, ConsoleUserInputRequester
+from streamlink_cli.constants import CONFIG_FILES, DEFAULT_STREAM_METADATA, PLUGINS_DIR, STREAM_SYNONYMS
+from streamlink_cli.output import FileOutput, PlayerOutput
+from streamlink_cli.utils import HTTPServer, ignored, progress, stream_to_url
 
 ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
 try:
@@ -106,7 +101,7 @@ def create_output(plugin):
 
             try:
                 namedpipe = NamedPipe(pipename)
-            except IOError as err:
+            except OSError as err:
                 console.exit("Failed to create pipe: {0}", err)
         elif args.player_http:
             http = create_http_server()
@@ -146,7 +141,7 @@ def create_http_server(host=None, port=0):
 def create_title(plugin=None):
     if args.title and plugin:
         title = LazyFormatter.format(
-            maybe_decode(args.title, get_filesystem_encoding()),
+            args.title,
             title=lambda: plugin.get_title() or DEFAULT_STREAM_METADATA["title"],
             author=lambda: plugin.get_author() or DEFAULT_STREAM_METADATA["author"],
             category=lambda: plugin.get_category() or DEFAULT_STREAM_METADATA["category"],
@@ -224,7 +219,7 @@ def output_stream_http(plugin, initial_streams, external=False, port=0):
                     sleep(10)
                     continue
             except PluginError as err:
-                log.error(u"Unable to fetch new streams: {0}".format(err))
+                log.error(f"Unable to fetch new streams: {err}")
                 continue
 
             try:
@@ -285,7 +280,7 @@ def open_stream(stream):
     try:
         log.debug("Pre-buffering 8192 bytes")
         prebuffer = stream_fd.read(8192)
-    except IOError as err:
+    except OSError as err:
         stream_fd.close()
         raise StreamError("Failed to read data from stream: {0}".format(err))
 
@@ -317,7 +312,7 @@ def output_stream(plugin, stream):
 
     try:
         output.open()
-    except (IOError, OSError) as err:
+    except OSError as err:
         if isinstance(output, PlayerOutput):
             console.exit("Failed to start player: {0} ({1})",
                          args.player, err)
@@ -374,7 +369,7 @@ def read_stream(stream, output, prebuffer, chunk_size=8192):
 
             try:
                 output.write(data)
-            except IOError as err:
+            except OSError as err:
                 if is_player and err.errno in ACCEPTABLE_ERRNO:
                     log.info("Player closed")
                 elif is_http and err.errno in ACCEPTABLE_ERRNO:
@@ -383,7 +378,7 @@ def read_stream(stream, output, prebuffer, chunk_size=8192):
                     console.exit("Error when writing to output: {0}, exiting", err)
 
                 break
-    except IOError as err:
+    except OSError as err:
         console.exit("Error when reading from stream: {0}, exiting", err)
     finally:
         stream.close()
@@ -472,7 +467,7 @@ def fetch_streams_with_retry(plugin, interval, count):
     try:
         streams = fetch_streams(plugin)
     except PluginError as err:
-        log.error(u"{0}".format(err))
+        log.error(err)
         streams = None
 
     if not streams:
@@ -488,7 +483,7 @@ def fetch_streams_with_retry(plugin, interval, count):
         except FatalPluginError:
             raise
         except PluginError as err:
-            log.error(u"{0}".format(err))
+            log.error(err)
 
         if count > 0:
             attempts += 1
@@ -586,7 +581,7 @@ def handle_url():
     except NoPluginError:
         console.exit("No plugin can handle URL: {0}", args.url)
     except PluginError as err:
-        console.exit(u"{0}", err)
+        console.exit("{0}", err)
 
     if not streams:
         console.exit("No playable streams found on this URL: {0}", args.url)
@@ -611,12 +606,16 @@ def handle_url():
         else:
             console.exit("{0}.\n       Available streams: {1}",
                          err, validstreams)
+    elif console.json:
+        console.msg_json(dict(plugin=plugin.module, streams=streams))
+    elif args.stream_url:
+        try:
+            console.msg("{0}", streams[list(streams)[-1]].to_manifest_url())
+        except TypeError:
+            console.exit("The stream specified cannot be translated to a URL")
     else:
-        if console.json:
-            console.msg_json(dict(streams=streams, plugin=plugin.module))
-        else:
-            validstreams = format_valid_streams(plugin, streams)
-            console.msg("Available streams: {0}", validstreams)
+        validstreams = format_valid_streams(plugin, streams)
+        console.msg("Available streams: {0}", validstreams)
 
 
 def print_plugins():
@@ -666,10 +665,10 @@ def setup_args(parser, config_files=[], ignore_unknown=False):
         args.url = args.url_param
 
 
-def setup_config_args(parser):
+def setup_config_args(parser, ignore_unknown=False):
     config_files = []
 
-    if args.url:
+    if streamlink and args.url:
         with ignored(NoPluginError):
             plugin = streamlink.resolve_url(args.url)
             config_files += ["{0}.{1}".format(fn, plugin.module) for fn in CONFIG_FILES]
@@ -684,7 +683,7 @@ def setup_config_args(parser):
             break
 
     if config_files:
-        setup_args(parser, config_files)
+        setup_args(parser, config_files, ignore_unknown=ignore_unknown)
 
 
 def setup_console(output):
@@ -692,8 +691,7 @@ def setup_console(output):
     global console
 
     # All console related operations is handled via the ConsoleOutput class
-    console = ConsoleOutput(output, streamlink)
-    console.json = args.json
+    console = ConsoleOutput(output, args.json)
 
     # Handle SIGTERM just like SIGINT
     signal.signal(signal.SIGTERM, signal.default_int_handler)
@@ -734,15 +732,6 @@ def setup_http_session():
     if args.http_timeout:
         streamlink.set_option("http-timeout", args.http_timeout)
 
-    if args.http_cookies:
-        streamlink.set_option("http-cookies", args.http_cookies)
-
-    if args.http_headers:
-        streamlink.set_option("http-headers", args.http_headers)
-
-    if args.http_query_params:
-        streamlink.set_option("http-query-params", args.http_query_params)
-
 
 def setup_plugins(extra_plugin_dir=None):
     """Loads any additional plugins."""
@@ -830,6 +819,8 @@ def setup_options():
 
     if args.rtmp_rtmpdump:
         streamlink.set_option("rtmp-rtmpdump", args.rtmp_rtmpdump)
+    elif args.rtmpdump:
+        streamlink.set_option("rtmp-rtmpdump", args.rtmpdump)
 
     if args.rtmp_timeout:
         streamlink.set_option("rtmp-timeout", args.rtmp_timeout)
@@ -852,10 +843,19 @@ def setup_options():
         streamlink.set_option("ffmpeg-verbose", args.ffmpeg_verbose)
     if args.ffmpeg_verbose_path:
         streamlink.set_option("ffmpeg-verbose-path", args.ffmpeg_verbose_path)
+    if args.ffmpeg_fout:
+        streamlink.set_option("ffmpeg-fout", args.ffmpeg_fout)
     if args.ffmpeg_video_transcode:
         streamlink.set_option("ffmpeg-video-transcode", args.ffmpeg_video_transcode)
     if args.ffmpeg_audio_transcode:
         streamlink.set_option("ffmpeg-audio-transcode", args.ffmpeg_audio_transcode)
+    if args.ffmpeg_copyts:
+        streamlink.set_option("ffmpeg-copyts", args.ffmpeg_copyts)
+    if args.ffmpeg_start_at_zero:
+        streamlink.set_option("ffmpeg-start-at-zero", args.ffmpeg_start_at_zero)
+
+    if args.mux_subtitles:
+        streamlink.set_option("mux-subtitles", args.mux_subtitles)
 
     streamlink.set_option("subprocess-errorlog", args.subprocess_errorlog)
     streamlink.set_option("subprocess-errorlog-path", args.subprocess_errorlog_path)
@@ -868,9 +868,24 @@ def setup_plugin_args(session, parser):
     plugin_args = parser.add_argument_group("Plugin options")
     for pname, plugin in session.plugins.items():
         defaults = {}
+        group = plugin_args.add_argument_group(pname.capitalize())
+
         for parg in plugin.arguments:
-            plugin_args.add_argument(parg.argument_name(pname), **parg.options)
-            defaults[parg.dest] = parg.default
+            if not parg.is_global:
+                group.add_argument(parg.argument_name(pname), **parg.options)
+                defaults[parg.dest] = parg.default
+            else:
+                pargdest = parg.dest
+                for action in parser._actions:
+                    # find matching global argument
+                    if pargdest != action.dest:
+                        continue
+                    defaults[pargdest] = action.default
+
+                    # add plugin to global argument
+                    plugins = getattr(action, "plugins", [])
+                    plugins.append(pname)
+                    setattr(action, "plugins", plugins)
 
         plugin.options = PluginOptions(defaults)
 
@@ -879,21 +894,26 @@ def setup_plugin_options(session, plugin):
     """Sets Streamlink plugin options."""
     pname = plugin.module
     required = OrderedDict({})
+
     for parg in plugin.arguments:
-        if parg.options.get("help") != argparse.SUPPRESS:
+        if parg.options.get("help") == argparse.SUPPRESS:
+            continue
+
+        value = getattr(args, parg.dest if parg.is_global else parg.namespace_dest(pname))
+        session.set_plugin_option(pname, parg.dest, value)
+
+        if not parg.is_global:
             if parg.required:
                 required[parg.name] = parg
-            value = getattr(args, parg.namespace_dest(pname))
-            session.set_plugin_option(pname, parg.dest, value)
             # if the value is set, check to see if any of the required arguments are not set
             if parg.required or value:
                 try:
                     for rparg in plugin.arguments.requires(parg.name):
                         required[rparg.name] = rparg
                 except RuntimeError:
-                    log.error("{0} plugin has a configuration error and the arguments "
-                              "cannot be parsed".format(pname))
+                    log.error(f"{pname} plugin has a configuration error and the arguments cannot be parsed")
                     break
+
     if required:
         for req in required.values():
             if not session.get_plugin_option(pname, req.dest):
@@ -960,10 +980,13 @@ def check_version(force=False):
 
 
 def setup_logging(stream=sys.stdout, level="info"):
-    fmt = ("[{asctime},{msecs:03.0f}]" if level == "trace" else "") + "[{name}][{levelname}] {message}"
-    logger.basicConfig(stream=stream, level=level,
-                       format=fmt, style="{",
-                       datefmt="%H:%M:%S")
+    logger.basicConfig(
+        stream=stream,
+        level=level,
+        style="{",
+        format=("[{asctime}]" if level == "trace" else "") + "[{name}][{levelname}] {message}",
+        datefmt="%H:%M:%S" + (".%f" if level == "trace" else "")
+    )
 
 
 def main():
@@ -971,6 +994,8 @@ def main():
     parser = build_parser()
 
     setup_args(parser, ignore_unknown=True)
+    # call argument set up as early as possible to load args from config files
+    setup_config_args(parser, ignore_unknown=True)
 
     # Console output should be on stderr if we are outputting
     # a stream to stdout.
@@ -1001,7 +1026,7 @@ def main():
     check_root()
     log_current_versions()
 
-    if args.version_check or (not args.no_version_check and args.auto_version_check):
+    if args.version_check or args.auto_version_check:
         with ignored(Exception):
             check_version(force=args.version_check)
 
