@@ -1,7 +1,7 @@
 import logging
 import re
+from urllib.parse import parse_qsl, urlparse
 
-from streamlink.compat import urlparse, parse_qsl
 from streamlink.plugin import Plugin
 from streamlink.plugin.api.utils import itertags
 from streamlink.stream import HLSStream
@@ -10,7 +10,7 @@ log = logging.getLogger(__name__)
 
 
 class Mjunoon(Plugin):
-    url_re = re.compile(r"""https?://(?:www.)?mjunoon.tv/""", re.VERBOSE)
+    url_re = re.compile(r"https?://(?:www\.)?mjunoon\.tv/")
 
     @classmethod
     def can_handle_url(cls, url):
@@ -27,8 +27,7 @@ class Mjunoon(Plugin):
                 for key, url in parse_qsl(urlparts.query):
                     if key == "streamUrl":
                         i += 1
-                        for s in HLSStream.parse_variant_playlist(self.session, url, params=dict(id=i), verify=False).items():
-                            yield s
+                        yield from HLSStream.parse_variant_playlist(self.session, url, params=dict(id=i), verify=False).items()
 
 
 __plugin__ = Mjunoon
