@@ -1,16 +1,21 @@
+import json
+import logging
 import re
 
 from streamlink.plugin import Plugin
-from streamlink.plugin.api import http, validate
-from streamlink.stream import RTMPStream
+from streamlink.stream import HLSStream, RTMPStream
+
+log = logging.getLogger(__name__)
 
 PAGE_URL = "https://www.tigerdile.com/stream/"
 ROOT_URL = "rtmp://stream.tigerdile.com/live/{0}"
+API_URL = "https://api.tigerdile.com/video?key={channel}"
+HLS_URL = "https://outbound.tigerdile.com/{channel}/index.m3u8"
 STREAM_TYPES = ["rtmp"]
 
 _url_re = re.compile(r"""
     https?://(?:www|sfw)\.tigerdile\.com
-    \/stream\/(.*)\/""", re.VERBOSE)
+    \/stream\/(.+)\/?""", re.VERBOSE)
 
 
 class Tigerdile(Plugin):
@@ -21,6 +26,18 @@ class Tigerdile(Plugin):
     def _get_streams(self):
         res = self.url
         streamname = _url_re.search(res).group(1)
+
+        ci = self.session.http.get(API_URL.format(channel=streamname))
+        api_json = json.loads(ci.text)
+
+        if not api_json or len(api_json) == 0:
+            log.error("The channel {0} does not exist or is marked private".format(streamname))
+            return
+
+        if not api_json[0]["online"]:
+            log.error("The channel {0} is not online".format(streamname))
+            return
+
         streams = {}
         stream = RTMPStream(self.session, {
             "rtmp": ROOT_URL.format(streamname),
@@ -33,6 +50,9 @@ class Tigerdile(Plugin):
         })
         streams["live"] = stream
 
+        stream_hls = HLSStream(self.session, HLS_URL.format(channel=streamname))
+        streams["live_hls"] = stream_hls
+
         return streams
 
 
